/*****************************************************************
**	regcards.c
*****************************************************************/

# include <stdio.h>
# include <assert.h>

# include "config.h"
# include "vcl.h"
# include "vocab.h"
#define extern
# include "regcards.h"
#undef extern

/*****************************************************************
**	len_slcnt
**	calculate the length of the largest substring and the
**	number of substrings (slashes)
**	A substring is delimited by a space surounded slash like
**		"substring one / substring two"
*****************************************************************/
static	int	len_slcnt (const char *str, int *nslash)
{
	int	len;
	int	maxlen;
	const	char	*p;
	const	char	*lastp = "";

	assert ( str != NULL );
	assert ( nslash != NULL );

	len = maxlen = 0;
	*nslash = 0;
	for ( p = str; *p; p++ )
	{
		if ( *p == '/' && *lastp == ' ' )
		{
			(*nslash)++;
			maxlen = max (maxlen, len);
			len = 0;
		}
		lastp = p;
		len++;
	}

	return max (len, maxlen);
}

/*****************************************************************
**	print_roffds()
*****************************************************************/
#define	vmove(a)	(((a)/2.0)-0.25)
static	int	print_roffds (FILE *out, const char *str, int nr, const char *var)
{
	int	len;
	int	size;
	int	nslash;
	const	char	*p;
	const	char	*lastp;

	len = len_slcnt (str, &nslash);
	size = (len / 20) * 2;

	len = 0;
	len += fprintf (out, ".ds %s%d \"", var, nr);
	if ( size )
		len += fprintf (out, "\\s-%d", size);
	if ( nslash )
	{
		len += fprintf (out, "\\\\v\'-%0.1fv\'", vmove (nslash));	/* move up a bit */
		len += fprintf (out, "\\\\kX");			/* mark horizontal pos */
	}
	lastp = "";
	for ( p = str; *p; p++ )
	{
		if ( *p == '/' && *lastp == ' ' )
		{
			len += fprintf (out, "\\\\h\'|\\\\nXu\'\\\\v\'1v\'");
			do
				lastp = p;
			while ( *++p == ' ' );
			p--;
		}
		else
			len += fprintf (out, "%c", *p);
		lastp = p;
	}
	if ( nslash )
	{
		len += fprintf (out, "\\\\v\'-%dv\'", nslash);	/* restore substring vertical movement */
		len += fprintf (out, "\\\\v\'%0.1fv\'", vmove (nslash));	/* restore the move up */
	}
	if ( size )
		len += fprintf (out, "\\s+%d", size);
	len += fprintf (out, "\n");

	return len;
}


static	int	print_cardline (FILE *out, int ncol, int nr, int dir)
{
	int	c;
	int	i;
	int	bytes = 0;

	assert ( dir == 1 || dir == -1 );

	i = nr;
	if ( dir == -1 )
		i += ncol-1;
	
	for ( c = 0; c < ncol; c++ )
	{
		bytes += fprintf (out, "\t\\*[%s%d]", dir == 1 ? "vocab": "trans", i);
		i += dir;
	}
	bytes += fprintf (out, "\n.br\n");

	return	bytes;
}

FILE	*cardstream (void)
{
	FILE	*fp;
#if defined(GROFF_PATH)
	char	str[127+1];

	snprintf (str, sizeof str, "%s -k -Tps -P-pa4", GROFF_PATH);
	if ( (fp = popen (str, "w")) == NULL )
		error ("Can't run groff\n");
#else
	fp = stdout;
#endif
	return fp;
}

int	print_cardheader (FILE *out)
{
	int	n = 0;
	
	n += fprintf (out, ".pl 29.7c\n");
	n += fprintf (out, ".ll 21c\n");
	n += fprintf (out, ".po 0\n");
	n += fprintf (out, ".in 0\n");
	n += fprintf (out, ".ps %d\n", ps);
	n += fprintf (out, ".vs %d\n", vs);

	return n;
}

int	print_cardwords (FILE *out, vc_t *v1, vc_t *v2, int nr)
{
	int	bytes = 0;

	if ( v1 )
	{
		bytes += print_roffds (out, v1->word, nr, "vocab");
		bytes += print_roffds (out, v1->trans, nr, "trans");
	}

	return bytes;
}

int	print_cardpage (FILE *out, int nrow, int ncol, int nr, int dir)
{
	float	rdiff = (29.7 - 0.5) / nrow;
	float	cdiff = (21 - 0.5) / ncol;
	int	r;
	int	i;
	int	bytes = 0;

	assert ( dir == 1 || dir == -1 );

	if ( verbose > 1 )
	{
		bytes += fprintf (stderr, "rdiff = %.2f\n", rdiff);
		bytes += fprintf (stderr, "cdiff = %.2f\n", cdiff);
	}

	// print column marker
	bytes += fprintf (out, ".ta T %.2fcC\n", cdiff);
	for ( i = 1; i < ncol; i++ )
		bytes += fprintf (out, "\t\\L\'%.2fc\'\\v\'%.2fc\'", 29.7-0.5, -(29.7-0.5));
	bytes += fprintf (out, "\n.br\n");

	bytes += fprintf (out, ".ta %.2fcC T %.2fcC\n", cdiff/2, cdiff);
	i = nr - nrow * ncol;
	for ( r = 0; r < nrow; r++ )
	{
		if ( r )
		{
			bytes += fprintf (out, ".sp |%.2fc\n", (r-1) * rdiff + rdiff);
			bytes += fprintf (out, "\\l\'%.2fc\'\n.br\n", 21 - 0.5);
		}
		bytes += fprintf (out, ".sp |%.2fc\n", r * rdiff + rdiff/2);
		bytes += print_cardline (out, ncol, i, dir);
		i += ncol;
	}

	return bytes;
}

