/*****************************************************************
**
**	(@)#	vcl.c	vocabular list data structure & functions
**
**	Copyright (c) Feb 2017, Holger Zuleger HZNET. All rights reserved.
**
**	This software is open source
**
*****************************************************************/
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include <ctype.h>
# include <assert.h>

# include "config.h"
#define extern
# include "vcl.h"
#undef extern


#define	skipline(str)	(	((str)[0] == '#') || \
				(isspace((str)[0])) ||  \
				((str)[0] == '-' && (str)[1] == '-' && (str)[2] == '-') \
			)

/*****************************************************************
**
**	file2vcl ( vcl, inverse, fp)
**
**	read the file into a vocabular list
**
**	If inverse is set, than word and translation is changed
**
**	returns the number of lines read
**
*****************************************************************/
int	file2vcl (vcl_t *vcl, int inverse, FILE *fp)
{
	char	line[511+1];
	char	*p;
	char	*new;
	int	lnr;

	lnr = 0;
	while ( fgets (line, sizeof line, fp) )
	{
		lnr++;
			/* check line length */
		for ( p = line; *p && *p != '\n'; p++ )
			;
		if ( *p == '\0' )	/* no linefeed ? */
		{
			int	c;

			fprintf (stderr, "Line %d too long. Skipped!\n", lnr);
			/* eat up the rest of the line */
			while ( (c = fgetc (fp)) != EOF && c != '\n' )
				;
			continue;
		}

		if ( skipline (line) )	/* skip empty and comment lines */
			continue;

			/* remove trailing white space */
		while ( p > line && isspace (*p) )
			p--;
		*++p = '\0';

			/* find first ':' or <tab> in the line */
		for ( p = line; *p != ':' && *p != '\t' && *p; p++ )
			;
		if ( *p == ':' || *p == '\0' )	/* skip tagged lines like "Date: .." */
			continue;		/* .. or lines w/o a <tab> */
		/* 'p' points to first <tab> found now */

		new = strdup (line);	/* duplicate line on heap memory */
		p = new + (p - line);	/* set 'p' to the same position again */
		*p++ = '\0';		/* mark end of "word" string */
		while ( isspace (*p) )	/* skip tab and space chars */
			p++;

		if ( !inverse )
		{
			vcl->list[vcl->n].word = new;
			vcl->list[vcl->n].trans = p;
		}
		else	/* flip word with translation */
		{
			vcl->list[vcl->n].word = p;
			vcl->list[vcl->n].trans = new;
		}
		vcl->n++;
		if ( vcl->n >= MAXVC )
			error ("Maximum number of vocables reached\n");

		if ( verbose > 2 )
			fputs (line, stderr);
	}

	return lnr;
}
